@@ -1,12 +1,12 @@
-;; aa-tree.scm - AA tree implementation for guile
+;; aa-tree.scm - core code file for the Guile AA Tree module
 ;; Copyright (C) 2017  Christopher Howard
 
-;; This program is free software: you can redistribute it and/or modify
+;; Guile AA Tree is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation, either version 3 of the License, or
 ;; (at your option) any later version.
 
-;; This program is distributed in the hope that it will be useful,
+;; Guile AA Tree is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
@@ -25,6 +25,7 @@
             aa-map-keys
             aa-map-reverse
             aa-map-values
+            aa-modify
             aa-search
             aa-search-wrapped
             aa-to-list
@@ -416,3 +417,12 @@ before the node is added."
                       ((equal? '_ (first (second lst))) (cdr (second lst)))
                       (else (list-to-aa-tree #nil (second lst)))))
           (drop lst 2)))))
+
+(define (aa-modify tree key fn)
+  "Uses aa-search to search for a node in TREE, then feeds the value
+of that node to FN, and then inserts the node back into TREE, but
+containing the new value returned from FN. This is an immutable
+operation provided you don't mutate the object returned from FN."
+  (aa-insert tree
+             key
+             (fn (aa-search tree key))))
